 /**
  * MyStringVector
  *
  * Realizza un vettore di stringhe.
  * Usa come struttura dati un array riempito parzialmente che viene dinamicamente ridimensionato in caso di necessita'.
  *
  * @version 3-Nov-2007
  * @author Adriano Luchetta
  *
  */
  
 public class MyStringVector
{
   private final int INITIAL_CAPACITY = 1;

   private String[] v;
   private int vSize;
   
   /**
      Costruisce un vettore vuoto con un array interno di dimensione 1.
   */
   public MyStringVector()
   {
      v = new String[INITIAL_CAPACITY];
      vSize = 0;
   }
   
   /**
       Se l'elemento specificato vale null, aggiunge l'elemento specificato in coda a questo vettore, altrimenti ritorna senza fare niente.
       @param s l'elemento da aggiungere a questo vettore
   */
   public void add(String s)
   {
      if (s == null)
         return;

      if (vSize >= v.length)
         resize();
         
      v[vSize] = s;
      vSize++;
   }
   
   /**
       Se 0 <= index <= size(), inserisce l'elemento specificato nella posizione specificata in questo vettore.
       Sposta a destra l'elemento attualmente in quella posizione (se alcuno) e ogni elemento successivo (aggiunge uno al loro indice).
       Se index < 0 0 index > size() ritorna senza fare niente. Ritorna senza fare niente anche se s vale null.
       @param index indice al quale l'elemento specificato deve essere inserito
       @param s l'elemento da aggiungere a questo vettore
   */
   public void add(int index, String s)
   {
      if (index < 0 || index > vSize || s == null)
         return;

      if (vSize >= v.length)
         resize();
         
      for (int i = vSize; i > index; i--)
         v[i] = v[i - 1];
         
      v[index] = s;
      
      vSize++;
   }
   
   /**
      Restituisce la capacita' del vettore.
      @return la dimensione dell'array interno.
   */
   public int capacity()
   {
      return v.length;
   }

   /**
      Restituisce tue se questo vettore contiene l'elemento specificato.
      @param s elemento la cui presenza in questo vettore e' da verificare
      @return true se l'elemento specificato e' contenuto in questo vettore, false altrimenti
   */
   public boolean contains(String s)
   {
      for (int i = 0; i < vSize; i++)
         if (v[i].equals(s))
            return true;
            
      return false;
   }
   
   /**
      Se 0<= index < size(), restituisce l'elemento all'indice specificato, altrimenti restituisce null.
      @param index un indice in questo vettore
      @return l'elemento all'indice specificato;
   */
   public String elementAt(int index)
   {
      if (index < 0 || index >=vSize)
         return null;

      return v[index];
   }
   
   /**
      restituisce l'indice della prima occorrenza dell'elemento specificato in questo vettore, o -1 se questo vettore non contiene l'elemento.
      @param s elemento da ricercare
      @return indice della prima occorrenza dell'elemento specificato
   */
   public int indexOf(String s)
   {
      for (int i = 0; i < vSize; i++)
         if (v[i].equals(s))
            return i;

      return -1;
   }
   
   /**
      Restituisce true se il vettore e' vuoto, true altrimenti
      @return true se e solo se il vettore non ha elementi, false altrimenti
   */
   public boolean isEmpty()
   {
      return vSize == 0;
   }
   
   /**
      Restituisce l'elemento massimo in ordine lessicografico di questo vettore. Se ci sono piu' massimi, restituisce uno dei massimi.
      @return uno dei massimi in senso lessicografico del vettore.
   */
   public String max()
   {
      String maxStr = v[0];
      for (int i = 1; i < vSize; i++)
         if (v[i].compareTo(maxStr) > 0)
            maxStr = v[i];
            
      return maxStr;
   }

   /**
      Rimuove tutti gli elementi di questo vettore e imposta la sua dimensione a zero.
   */
   public void makeEmpty()
   {
      v = new String[INITIAL_CAPACITY];
      vSize = 0;
   }


   /**
      Restituisce l'elemento minimo in ordine lessicografico di questo vettore. Se ci sono piu' minimi, restituisce uno dei minimi.
      @return uno dei minimi in senso lessicografico del vettore.
   */
   public String min()
   {
      String minStr = v[0];
      for (int i = 1; i < vSize; i++)
         if (v[i].compareTo(minStr) < 0)
            minStr = v[i];

      return minStr;
   }

   /**
      Se 0 <= index < size(), rimuove l'elemento all'indice specificato, restituendolo. Sposta a sinistra ogni elemento successivo (sottrae uno al loro indice).
      Se index < 0 o index >=size() restituisce null.
      @param index indice da rimuovere
      @return l'elemento rimosso all'indice specificato
   */
   public String remove(int index)
   {
      if (index < 0 || index >= vSize)
         return null;
         
      String tmp = v[index];
      
      for (int i = index; i < vSize - 1; i++)
         v[i] = v[i + 1];
         
      vSize--;
         
      return tmp;
   }

   /**
      Se 0 <= index < size(), sostituisce l'elemento alla posizione specificata in questo vettore con l'elemento specificato.
      Se index < 0 o index >=size() restituisce null.
      @param index indice dell'elemento da sostituire
      @param s elemento da memorizzare alla posizione specificata
      @return l'elemento precedentemente memorizzato alla posizione specificata.
   */
   public String set(int index, String s)
   {
      if (index < 0 || index >= vSize)
         return null;
         
      String tmp = v[index];
      v[index] = s;

      return tmp;
   }

   /**
      Restituisce il numero di componenti in questo vettore.
      @return il numero di componenti in questo vettore.
   */
   public int size()
   {
      return vSize;
   }
   
   /**
      Restituisce un array contenente tutti gli elementi in questo vettore nell'ordine corretto.
      @return un array contenente tutti gli elementi di questo vettore.
   */
   public String[] toArray()
   {
      String[] tmp = new String[vSize];
      for (int i = 0; i < vSize; i++)
         tmp[i] = v[i];
         
      return tmp;
   }
   
   /**
      Restituisce una stringa contenente gli elementi di questo array in ordine, un elemento per riga.
      @return una stringa contenente gli elementi di questo array in ordine, un elemento per riga.
   */
   public String toString()
   {
      String tmp = "";
      for (int i = 0; i < vSize; i++)
         tmp = tmp +  v[i] + "\n";
         
      return tmp;
   }
   
   /*
      ridimensionamento dinamico di questo vettore
   */
   private void resize()
   {
      String[] tmp = new String[2 * v.length];
      for (int i = 0; i < v.length; i++)
         tmp[i] = v[i];
         
      v = tmp;
   }
}